@@ -1,6 +1,6 @@
 package Sub::Spec::CmdLine;
 BEGIN {
-  $Sub::Spec::CmdLine::VERSION = '0.10';
+  $Sub::Spec::CmdLine::VERSION = '0.11';
 }
 # ABSTRACT: Access Perl subs via command line
 
@@ -278,10 +278,11 @@ sub run {
     require Getopt::Long;
 
     my %args = @_;
+    my $exit = $args{exit} // 1;
 
     my %opts = (format => undef, action => 'run');
     Getopt::Long::Configure("pass_through", "no_permute");
-    Getopt::Long::GetOptions(
+    my %getopts = (
         "--list|l"     => sub { $opts{action} = 'list'     },
         "--version|v"  => sub { $opts{action} = 'version'  },
         "--help|h|?"   => sub { $opts{action} = 'help'     },
@@ -292,24 +293,12 @@ sub run {
         "--pretty"     => sub { $opts{format} = 'pretty'   },
         "--nopretty"   => sub { $opts{format} = 'nopretty' },
     );
+    Getopt::Long::GetOptions(%getopts);
 
-    my $exit = $args{exit} // 1;
     my $subcmds = $args{subcommands};
     my $module;
     my $sub;
 
-    # handle --list
-    if ($opts{action} eq 'list') {
-        if ($subcmds) {
-            # XXX sort by category
-            for my $c (sort keys %$subcmds) {
-                my $sc = $subcmds->{$c};
-                say "  $c", ($sc->{summary} ? " - $sc->{summary}" : "");
-            }
-        }
-        if ($exit) { exit 0 } else { return 0 }
-    }
-
     # finding out which module/sub to use
     my $subcmdname;
     my $subcmd;
@@ -325,6 +314,77 @@ sub run {
         $sub    = $args{sub};
     }
 
+    # require module and get spec
+    my $spec;
+    if ($module && $sub) {
+        my $modulep = $args{module};
+        $modulep =~ s!::!/!g; $modulep .= ".pm";
+        if ($args{require} // 1) {
+            eval { require $modulep };
+            die $@ if $@;
+        }
+        no strict 'refs';
+        my $subs = \%{$module."::SUBS"};
+        $spec = $subs->{$sub};
+        die "Can't find spec for sub $module\::$sub\n" unless $spec;
+    }
+
+    # detect if we're being invoked for bash completion
+    if (defined $ENV{COMP_LINE}) {
+        {
+            eval { require Sub::Spec::BashComplete };
+            my $eval_err = $@;
+            if ($eval_err) {
+                $log->warn("Can't load Sub::Spec::BashComplete: $eval_err");
+                last;
+            }
+
+            my @general_opts;
+            for my $o (keys %getopts) {
+                $o =~ s/^--//;
+                my @o = split /\|/, $o;
+                for (@o) { push @general_opts, length > 1 ? "--$_" : "-$_" }
+            }
+
+            my $res = Sub::Spec::BashComplete::_parse_request();
+            my $words = $res->{words};
+            my $cword = $res->{cword};
+            my $word  = $words->[$cword] // "";
+
+            if ($spec) {
+                shift @$words;
+                $cword--;
+                print map {"$_\n"}
+                    Sub::Spec::BashComplete::bash_complete_spec_arg(
+                        $spec,
+                        {words=>$words, cword=>$cword},
+                    );
+                last;
+            }
+
+            # general options
+            print map {"$_\n"}
+                Sub::Spec::BashComplete::_complete_array(
+                $word,
+                \@general_opts
+            );
+        }
+
+        if ($exit) { exit 0 } else { return 0 }
+    }
+
+    # handle --list
+    if ($opts{action} eq 'list') {
+        if ($subcmds) {
+            # XXX sort by category
+            for my $c (sort keys %$subcmds) {
+                my $sc = $subcmds->{$c};
+                say "  $c", ($sc->{summary} ? " - $sc->{summary}" : "");
+            }
+        }
+        if ($exit) { exit 0 } else { return 0 }
+    }
+
     # handle --version
     if ($opts{action} eq 'version') {
         no strict 'refs';
@@ -339,18 +399,6 @@ sub run {
 
     my $cmd = $args{cmd} // $0;
 
-    # require module and get spec
-    my $modulep = $args{module};
-    $modulep =~ s!::!/!g; $modulep .= ".pm";
-    if ($args{require} // 1) {
-        eval { require $modulep };
-        die $@ if $@;
-    }
-    no strict 'refs';
-    my $subs = \%{$module."::SUBS"};
-    my $spec = $subs->{$sub};
-    die "Can't find spec for sub $module\::$sub\n" unless $spec;
-
     # handle general --help
     if ($opts{action} eq 'help') {
         if ($args{subcommands}) {
@@ -402,7 +450,7 @@ Sub::Spec::CmdLine - Access Perl subs via command line
 
 =head1 VERSION
 
-version 0.10
+version 0.11
 
 =head1 SYNOPSIS
 
@@ -522,7 +570,7 @@ and there is no data returned, this default_success_message is used. Example:
 
 =head2 run(%args)
 
-Run sub from the command line, which essentially comprises these
+Run subroutine(s) from the command line, which essentially comprises these
 steps:
 
 =over 4
@@ -573,6 +621,13 @@ If set to 0, do not try to require the module.
 
 =back
 
+run() can also perform completion for bash (if L<Sub::Spec::BashCompletion> is
+available). To get bash completion for your B<perlprog>, just type this in bash:
+
+ % complete -C /path/to/perlprog perlprog
+
+You can add that line in bash startup file (~/.bashrc, /etc/bash.bashrc, etc).
+
 =head1 SEE ALSO
 
 L<Sub::Spec>
